//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.access.api_1_svw_seekb_bss;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.access.*;
import sim.toolkit.*;
import javacard.framework.*;

import sim.test.util.*;
import sim.test.access.*;


public class API_1_SVW_SEEKB_BSS_1 extends TestToolkitApplet {

    SIMView theSimView;

    final static short BUFFER_LENGTH = (short)20;

    byte[] patt = new byte[BUFFER_LENGTH];
    byte[] nullBuffer = null;

    /**
     * The Constructor registers the application for the Event.
     */
    private API_1_SVW_SEEKB_BSS_1() {

        // Get a reference to the SIM View.
        theSimView = SIMSystem.getTheSIMView();
        
    }


    public static void install(byte[] bArray, short bOffset, byte bLength) {
        API_1_SVW_SEEKB_BSS_1 testApp = new API_1_SVW_SEEKB_BSS_1();

        // Register the applet with the JCRE.
        testApp.register(bArray, (short)(bOffset+1), bArray[bOffset]);

        // Initialise the data of the test applet.
        testApp.init();
    }


    /**
     * This method will be used to trigger the tests.
     * The application will be triggered by an Formatted SMS PP download event.
     * @param event 
     */
    public void processToolkit (byte event) throws ToolkitException {

        short result = 0;
        short pattOffset = 0;
        short pattLength = 0;
        byte mode = 0;

        if (event == EVENT_FORMATTED_SMS_PP_ENV) {

            /*------------------------------------------------------------------
             * TEST CASE 01: Errorcase, No EF selected
             *----------------------------------------------------------------*/
            try {
                pattOffset = 0;
                pattLength = 10;
                mode = SIMView.SEEK_FROM_BEGINNING_FORWARD;
                
                result = theSimView.seek(mode, patt, pattOffset, pattLength);

                reportTestFailure((byte)TestConstants.TESTCASE01);

            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE01, 
                        sve.getReason() == SIMViewException.NO_EF_SELECTED);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE01);
            }

            /*------------------------------------------------------------------
             * TEST CASE 02: Errorcase, Pattern not found
             *----------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_DF_SIMTEST);
                theSimView.select(TestConstants.FID_EF_LARU);
                patt[0] = (byte)0xDA;
                pattOffset = 0;
                pattLength = 1;
                mode = SIMView.SEEK_FROM_BEGINNING_FORWARD;
                
                result = theSimView.seek(mode, patt, pattOffset, pattLength);
                reportTestFailure((byte)TestConstants.TESTCASE02);

            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE02, 
                        sve.getReason() == SIMViewException.PATTERN_NOT_FOUND);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE02);
            }

            /*------------------------------------------------------------------
             * TEST CASE 03: Seek from beginning forward
             *----------------------------------------------------------------*/
            try {
                patt[0] = patt[1] = patt[2] = (byte)0x55;
                pattOffset = 0;
                pattLength = 3;
                mode = SIMView.SEEK_FROM_BEGINNING_FORWARD;
                
                result = theSimView.seek(mode, patt, pattOffset, pattLength);

                if (result==1)
                    reportTestSuccess((byte)TestConstants.TESTCASE03);
                else
                    reportTestFailure((byte)TestConstants.TESTCASE03);

            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE03);
            }

            /*------------------------------------------------------------------
             * TEST CASE 04: Seek from end backward
             *----------------------------------------------------------------*/
            try {
                mode = SIMView.SEEK_FROM_END_BACKWARD;
                
                result = theSimView.seek(mode, patt, pattOffset, pattLength);

                if (result==1)
                    reportTestSuccess((byte)TestConstants.TESTCASE04);
                else
                    reportTestFailure((byte)TestConstants.TESTCASE04);

            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE04);
            }

            /*------------------------------------------------------------------
             * TEST CASE 05: Seek from next forward
             *----------------------------------------------------------------*/
            try {
                patt[0] = patt[1] = patt[2] = (byte)0xAA;
                pattOffset = 0;
                pattLength = 3;
                mode = SIMView.SEEK_FROM_NEXT_FORWARD;
                
                result = theSimView.seek(mode, patt, pattOffset, pattLength);

                if (result==2)
                    reportTestSuccess((byte)TestConstants.TESTCASE05);
                else
                    reportTestFailure((byte)TestConstants.TESTCASE05);

            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE05);
            }

            /*------------------------------------------------------------------
             * TEST CASE 06: Errorcase, Last record, seek from next forward
             *----------------------------------------------------------------*/
            try {
                mode = SIMView.SEEK_FROM_NEXT_FORWARD;
                result = theSimView.seek(mode, patt, pattOffset, pattLength);
                reportTestFailure((byte)TestConstants.TESTCASE06);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE06, 
                        sve.getReason() == SIMViewException.PATTERN_NOT_FOUND);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE06);
            }

            /*------------------------------------------------------------------
             * TEST CASE 07: Seek from previous backward
             *----------------------------------------------------------------*/
            try {
                patt[0] = patt[1] = patt[2] = (byte)0x55;
                pattOffset = 0;
                pattLength = 3;
                mode = SIMView.SEEK_FROM_PREVIOUS_BACKWARD;
                
                result = theSimView.seek(mode, patt, pattOffset, pattLength);

                if (result==1)
                    reportTestSuccess((byte)TestConstants.TESTCASE07);
                else
                    reportTestFailure((byte)TestConstants.TESTCASE07);

            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE07);
            }

            /*------------------------------------------------------------------
             * TEST CASE 08: Errorcase, First record, seek from previous backward
             *----------------------------------------------------------------*/
            try {
                mode = SIMView.SEEK_FROM_PREVIOUS_BACKWARD;
                
                result = theSimView.seek(mode, patt, pattOffset, pattLength);

                reportTestFailure((byte)TestConstants.TESTCASE08);

            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE08, 
                        sve.getReason() == SIMViewException.PATTERN_NOT_FOUND);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE08);
            }

            /*------------------------------------------------------------------
             * TEST CASE 09: Errorcase, Pattern not found (out of reach)
             *----------------------------------------------------------------*/
            try {
                patt[0] = patt[1] = patt[2] = (byte)0x55;
                pattOffset = 0;
                pattLength = 3;
                mode = SIMView.SEEK_FROM_NEXT_FORWARD;
                
                result = theSimView.seek(mode, patt, pattOffset, pattLength);

                reportTestFailure((byte)TestConstants.TESTCASE09);

            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE09, 
                        sve.getReason() == SIMViewException.PATTERN_NOT_FOUND);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE09);
            }

            /*------------------------------------------------------------------
             * TEST CASE 10: Errorcase, Invalid mode
             *----------------------------------------------------------------*/
            try {
                mode = 4;
                
                try {
                    result = theSimView.seek(mode, patt, pattOffset, pattLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.INVALID_MODE)
                        UserException.throwIt((short)0);
                }

                mode = (byte)-1;
                
                try {
                    result = theSimView.seek(mode, patt, pattOffset, pattLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.INVALID_MODE)
                        UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)TestConstants.TESTCASE10);

            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE10);
            }

            /*------------------------------------------------------------------
             * TEST CASE 11: Errorcase, patt is null
             *----------------------------------------------------------------*/
            try {
                pattOffset = 0;
                pattLength = 3;
                mode = SIMView.SEEK_FROM_BEGINNING_FORWARD;

                result = theSimView.seek(mode, nullBuffer, pattOffset, pattLength);
                reportTestFailure((byte)TestConstants.TESTCASE11);

            } catch (NullPointerException npe) {
                    reportTestSuccess((byte)TestConstants.TESTCASE11);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE11);
            }

            /*------------------------------------------------------------------
             * TEST CASE 12: Errorcase, pattOffset < 0
             *----------------------------------------------------------------*/
            try {
                pattOffset = (short)-1;
                pattLength = 3;
                mode = SIMView.SEEK_FROM_BEGINNING_FORWARD;
                result = theSimView.seek(mode, patt, pattOffset, pattLength);
                reportTestFailure((byte)TestConstants.TESTCASE12);

            } catch (ArrayIndexOutOfBoundsException e) {
                reportTestSuccess((byte)TestConstants.TESTCASE12);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE12);
            }

            /*------------------------------------------------------------------
             * TEST CASE 13: Errorcase, pattLength < 0
             *----------------------------------------------------------------*/
            try {
                pattOffset = 0;
                pattLength = (short)-1;
                mode = SIMView.SEEK_FROM_BEGINNING_FORWARD;

                result = theSimView.seek(mode, patt, pattOffset, pattLength);
                reportTestFailure((byte)TestConstants.TESTCASE13);
            } catch (ArrayIndexOutOfBoundsException e) {
                reportTestSuccess((byte)TestConstants.TESTCASE13);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE13);
            }

            /*------------------------------------------------------------------
             * TEST CASE 14: Errorcase, pattLength > size of record
             *----------------------------------------------------------------*/
            try {
                patt[0] = patt[1] = patt[2] = patt[3] = patt[4] = (byte)0x55;
                pattOffset = 0;
                pattLength = 5;
                mode = SIMView.SEEK_FROM_BEGINNING_FORWARD;

                result = theSimView.seek(mode, patt, pattOffset, pattLength);

                reportTestFailure((byte)TestConstants.TESTCASE14);

            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE14, 
                        sve.getReason() == SIMViewException.OUT_OF_RECORD_BOUNDARIES);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE14);
            }

            /*------------------------------------------------------------------
             * TEST CASE 15: Errorcase, pattOffset + pattLength > patt.length
             *----------------------------------------------------------------*/
            try {
                pattOffset = 1;
                pattLength = (short)patt.length;

                result = theSimView.seek(mode, patt, pattOffset, pattLength);
                reportTestFailure((byte)TestConstants.TESTCASE15);

            } catch (ArrayIndexOutOfBoundsException e) {
                reportTestSuccess((byte)TestConstants.TESTCASE15);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE15);
            }

            /*--------------------------------------------------------------
             * TEST CASE 16: EF is not Linear Fixed
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_TNU);
                pattOffset = 0;
                pattLength = 3;
                mode = SIMView.SEEK_FROM_BEGINNING_FORWARD;

                try {
                    result = theSimView.seek(mode, patt, pattOffset, pattLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.FILE_INCONSISTENT)
                        UserException.throwIt((short)0);
                }

                theSimView.select(TestConstants.FID_EF_CNU);

                try {
                    result = theSimView.seek(mode, patt, pattOffset, pattLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.FILE_INCONSISTENT)
                        UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)TestConstants.TESTCASE16);

            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE16);
            }

            /*------------------------------------------------------------------
             * TEST CASE 17: Errorcase, Access condition not fulfilled
             *----------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_LNR);
                patt[0] = (byte)0x55;
                pattOffset = 0;
                pattLength = 1;
                mode = SIMView.SEEK_FROM_BEGINNING_FORWARD;

                result = theSimView.seek(mode, patt, pattOffset, pattLength);

                reportTestFailure((byte)TestConstants.TESTCASE17);

            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE17, 
                        sve.getReason() == SIMViewException.AC_NOT_FULFILLED);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE17);
            }

            /*------------------------------------------------------------------
             * TEST CASE 18: Errorcase, EF is invalidated
             *----------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_LARU);
                theSimView.invalidate();
                patt[0] = (byte)0x55;
                pattOffset = 0;
                pattLength = 1;
                mode = SIMView.SEEK_FROM_BEGINNING_FORWARD;

                try {
                    result = theSimView.seek(mode, patt, pattOffset, pattLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.INVALIDATION_STATUS_CONTRADICTION)
                        UserException.throwIt((short)0);
                } finally {
                    theSimView.rehabilitate();
                }

                reportTestSuccess((byte)TestConstants.TESTCASE18);

            } catch (Exception e){
                reportTestFailure((byte)TestConstants.TESTCASE18);
            }

        } // end if
    }
}
