//-----------------------------------------------------------------------------
//    API_2_TKR_CEVTB_1.java
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.toolkit.api_2_tkr_cevtb;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.toolkit.* ;
import javacard.framework.* ;
import sim.test.util.* ;

/**
 * Test Area : sim.toolkit.ToolkitRegistry.clearEvent(byte event) method
 * 
 * Applet is triggered on Install (Install), EVENT_FORMATTED_SMS_PP_ENV
 *
 */
public class API_2_TKR_CEVTB_1 extends TestToolkitApplet
{
	
	//Not allowed events for the ClearEvent() method
	private static byte[] NOT_ALLOWED_CLEAR_EVENTS = 
												{EVENT_MENU_SELECTION,
													EVENT_MENU_SELECTION_HELP_REQUEST,
														EVENT_TIMER_EXPIRATION,
															EVENT_STATUS_COMMAND};

	//Not supported events for the setEvent() method
	private static short NOT_SUPPORTED_SETEVENT_0 = 0;
	private static short NOT_SUPPORTED_SETEVENT_22 = 22;
	private static short NOT_SUPPORTED_SETEVENT_23 = 23;

	private static short SUPPORTED_SETEVENT_INTERVAL_MAX = 24;
	private static short SUPPORTED_SETEVENT_INTERVAL_MIN = -1;

	//Event value range
	private	static short MIN_EVENT_NB = -1;
	private	static short MAX_EVENT_NB = 127;

	//Indicate if that applet has already been triggered 1 time
	//-->all events have been cleared
	private	boolean eventsCleared = false;


    /**
     * Constructor of the applet
     */
    public API_2_TKR_CEVTB_1() 
    {
    }

    /**
     * Method called by the JCRE at the installation of the applet
     */
    public static void install(byte bArray[], short bOffset, byte bLength) 
    {
        // Create a new applet instance
        API_2_TKR_CEVTB_1 thisApplet = new API_2_TKR_CEVTB_1();

        // Register the new applet instance to the JCRE            
        thisApplet.register(bArray, (short)(bOffset+1), (byte) bArray[bOffset]);

        // Initialise the data of the test applet
        thisApplet.init();    
    }
    

	 /**
      * Method called by the processToolkit to know if event
	  * is an allowed event for the clearEvent() or setEvent() method
      */
    private boolean isEventAllowed(byte event)
	{
		boolean allowed = true;

		for(short i = ((short) (NOT_ALLOWED_CLEAR_EVENTS.length-1));
			((i > (short) -1) && (allowed));i--){
			allowed = (NOT_ALLOWED_CLEAR_EVENTS[i] != event);
		}
		return allowed;
	}

	 /**
      * Method called by the processToolkit to know if event
	  * is a supported event for the setEvent() method
	  */
    private boolean isEventSupported(byte event)
	{
		return 	((event != NOT_SUPPORTED_SETEVENT_0)
				&& (event != NOT_SUPPORTED_SETEVENT_22)
		      	&& (event != NOT_SUPPORTED_SETEVENT_23)
		      	&& (event >= SUPPORTED_SETEVENT_INTERVAL_MIN)
				&& (event <= SUPPORTED_SETEVENT_INTERVAL_MAX));			
						//check if event is not equals to
						//a not supported event (Interval of values)
	}

    /**
     * Method called by the SIM Toolkit Framework
     */
    public void processToolkit(byte event) 
    {
        // Result of tests
        boolean bRes ;
        
        // Number of tests
        byte testCaseNb = (byte) 0x04;
        
        // Get the system instance of the ProactiveHandler class
        ProactiveHandler proHdlr = ProactiveHandler.getTheHandler() ;

        ProactiveResponseHandler proRespHdlr = null ;


			// -----------------------------------------------------------------
			// Test Case 1 :For events ranging from 1 to 127 and for
			//				EVENT_UNRECOGNIZED_ENVELOPE (-1),  except
			//				those that aren't allowed, (EVENT_MENU_SELECTION,
			//				EVENT_MENU_SELECTION_HELP_REQUEST,
			//				EVENT_TIMER_EXPIRATION, EVENT_STATUS_COMMAND, the
			//				applet calls :
			//				1 - clearEvent() method
			//				2 - isEventSet() method
			//					Shall return false each time
			// -----------------------------------------------------------------
        
			testCaseNb = (byte) 1 ;

			byte eventID;
			bRes = false;
        
				for(short i = MIN_EVENT_NB;i <= MAX_EVENT_NB;i++){
					eventID = (byte) i;
					bRes = false;
			// -----------------------------------------------------------------
			//				1 - clearEvent() method
			//					No exception is thrown each time
			// -----------------------------------------------------------------
					if (this.isEventAllowed(eventID)){	
						try{
							obReg.clearEvent(eventID);
							bRes = true;
						}
						catch (Exception e){ bRes = false; }
						
			// -----------------------------------------------------------------
			//				2 - isEventSet() method
			//					Shall return false each time
			// -----------------------------------------------------------------
						if(obReg.isEventSet(eventID)){
						//A cleared event should
						//not be set anymore
							bRes = false;
							break;
						}
					}
				}
			 
		
			reportTestOutcome(testCaseNb,bRes) ;


			// -----------------------------------------------------------------
			// Test Case 2 :
			//				1 - For each ALLOWED and SUPPORTED events, 
			//					the applet calls setEvent() method
			//					No exception shall be thrown
			//
			//				2 - For events ranging from 1 to 127 and for 
			//					EVENT_UNRECOGNIZED_ENVELOPE (-1) except
			//					those that aren't allowed, the applet calls :
			//						2.1 -	clearEvent() method
			//								No exception shall be thrown
			//						2.2 -	isEventSet() method
			//								Shall return false.
			// -----------------------------------------------------------------
        
			testCaseNb = (byte) 2 ;
        
			bRes = false;

			// -----------------------------------------------------------------
			//				1 - For each ALLOWED and SUPPORTED events, 
			//					the applet calls setEvent() method
			//					No exception shall be thrown
			// -----------------------------------------------------------------
			try {            
            
				for(short i = MIN_EVENT_NB;i <= MAX_EVENT_NB;i++){
					eventID = (byte) i;

					if ((this.isEventAllowed(eventID))
						&& (this.isEventSupported(eventID))){
						obReg.setEvent(eventID);
					}
				}
				bRes = true;
			} 
			catch (Exception e) {	//No exception should be thrown
				bRes = false ;
			}

			// -----------------------------------------------------------------
			//				2 - For events ranging from 1 to 127 and for 
			//					EVENT_UNRECOGNIZED_ENVELOPE (-1) except
			//					those that aren't allowed, the applet calls :
			// -----------------------------------------------------------------
			if (bRes){	
				try {
					for(short i = MIN_EVENT_NB;i <= MAX_EVENT_NB;i++){
						eventID = (byte) i;

			// -----------------------------------------------------------------
			//						2.1 -	clearEvent() method
			//								No exception shall be thrown
			// -----------------------------------------------------------------
						if (this.isEventAllowed(eventID)){
							obReg.clearEvent(eventID);

			// -----------------------------------------------------------------
			//						2.2 -	isEventSet() method
			//								Shall return false
			// -----------------------------------------------------------------
							if(obReg.isEventSet(eventID)){
								bRes = false;
								break;
							}
						}
					}
				}
				catch (Exception e) {	
					bRes = false ;
				}
			}

			reportTestOutcome(testCaseNb, bRes) ;


			// -----------------------------------------------------------------
			// Test Case 3 :	
			//				For each event among:
			//				EVENT_MENU_SELECTION, EVENT_MENU_SELECTION_HELP_REQUEST, 
			//				EVENT_TIMER_EXPIRATION, EVENT_STATUS_COMMAND :
			//				The applet calls clearEvent(event) method
			//				Each time, clearEvent shall throw a Toolkit Exception 
			//				with reason EVENT_NOT_ALLOWED
			// -----------------------------------------------------------------
        
			testCaseNb = (byte) 3 ;
			bRes = false;

			try {
				obReg.clearEvent(NOT_ALLOWED_CLEAR_EVENTS
											[(short)(NOT_ALLOWED_CLEAR_EVENTS.length - 1)]);
				}
			catch (ToolkitException e) {	//Only EVENT_NOT_ALLOWED
											//should be thrown
				bRes = (e.getReason()==ToolkitException.EVENT_NOT_ALLOWED);
			}
			catch (Exception e){
				bRes = false;
			}

			for(short i = (short) (NOT_ALLOWED_CLEAR_EVENTS.length - 2);
					((i > (short) -1) && (bRes));i--){
				bRes = false;
				try {
					obReg.clearEvent(NOT_ALLOWED_CLEAR_EVENTS[i]);
					}
				catch (ToolkitException e) {	//Only EVENT_NOT_ALLOWED
												//should be thrown
					bRes = (e.getReason()==ToolkitException.EVENT_NOT_ALLOWED);
				}
				catch (Exception e){
				bRes = false;
				}
			}	
			reportTestOutcome(testCaseNb, bRes) ;

			// --------------------------------------------------------------------
		    // Test Case 4 :	
			//				Checking applet isn't triggered 
			//				by an ENVELOPE(SMS-PP DOWNLOAD) command
			// --------------------------------------------------------------------
			testCaseNb = (byte) 4 ;
			bRes = !eventsCleared;
			eventsCleared = true;
			reportTestOutcome(testCaseNb, bRes) ;
	}
}
