//-----------------------------------------------------------------------------
//    API_2_PRH_GTCS_1.java
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.toolkit.api_2_prh_gtcs;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.toolkit.* ;
import javacard.framework.* ;
import sim.test.util.* ;

/**
 * Test Area : sim.toolkit.ProactiveResponseHandler.getTextStringCodingScheme() method
 * 
 * Applet is triggered on Install (Install), EVENT_FORMATTED_SMS_PP_ENV
 *
 * @version 0.0.2 - 07/12/00
 * @author 3GPP T3 SWG API
 *
 * @see sim.toolkit.ProactiveResponseHandler
 */
public class API_2_PRH_GTCS_1 extends TestToolkitApplet
{


    private byte[] TEXT = {(byte)'T',(byte)'e',(byte)'x',(byte)'t'} ;

    /**
     * Constructor of the applet
     */
    public API_2_PRH_GTCS_1() 
    {
    }


    /**
     * Method called by the JCRE at the installation of the applet
     */
    public static void install(byte bArray[], short bOffset, byte bLength) 
    {
        // Create a new applet instance
        API_2_PRH_GTCS_1 thisApplet = new API_2_PRH_GTCS_1();

        // Register the new applet instance to the JCRE            
        thisApplet.register(bArray, (short)(bOffset+1), bArray[bOffset]);

        // Initialise the data of the test applet
        thisApplet.init();    
    }
    

    /**
       * Method called by the SIM Toolkit Framework
     */
    public void processToolkit(byte event) 
    {
        // Result of tests
        boolean bRes ;
        
        // Number of tests
        byte testCaseNb = (byte) 0x00 ;
        
        // Get the system instance of the ProactiveHandler class
        ProactiveHandler proHdlr = ProactiveHandler.getTheHandler() ;

        ProactiveResponseHandler proRespHdlr = null ;
    
        short result = (short) 0 ;
        
        // --------------------------------------------
        // Test Case 1 : Unavailable Text String TLV
        
        testCaseNb = (byte) 1 ;
        
        bRes = false ;
        try {
            
            // Build and send a proactive command
            proHdlr.initDisplayText((byte)0x00, DCS_8_BIT_DATA, TEXT, (short)0, (short)TEXT.length) ;
        
            proHdlr.send() ;
            
            // Get the response
            proRespHdlr = ProactiveResponseHandler.getTheHandler() ;
            
            try {
                
                // getTextStringCodingScheme() with unavailable Text String TLV
                result = proRespHdlr.getTextStringCodingScheme() ;
                
            } catch (ToolkitException e) {
                
                if (e.getReason() == ToolkitException.UNAVAILABLE_ELEMENT) {
                    bRes = true ;
                }
            }
            
        } catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 2 : Null Text String TLV
        
        testCaseNb = (byte) 2 ;
        
        bRes = false ;
        try {
            
            // Build and send a GET INPUT proactive command
            proHdlr.initGetInput((byte)0x00, DCS_8_BIT_DATA, TEXT, (short)0, (short)TEXT.length, (short)0x00, (short)0xFF) ;
        
            proHdlr.send() ;
            
            // Get the response
            proRespHdlr = ProactiveResponseHandler.getTheHandler() ;
            
            try {
                
                // getTextStringCodingScheme() with null Text String TLV
                result = proRespHdlr.getTextStringCodingScheme() ;
                
            } catch (ToolkitException e) {
                
                if (e.getReason() == ToolkitException.OUT_OF_TLV_BOUNDARIES) {
                    bRes = true ;
                }
            }
                
        } catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 3 : Successfull call
        
        testCaseNb = (byte) 3 ;
        
        bRes = false ;
        try {
            
            // Build and send a GET INPUT proactive command
            proHdlr.initGetInput((byte)0x00, DCS_8_BIT_DATA, TEXT, (short)0, (short)TEXT.length, (short)0x00, (short)0xFF) ;
        
            proHdlr.send() ;
            
            // Get the response
            proRespHdlr = ProactiveResponseHandler.getTheHandler() ;
            
            result = proRespHdlr.getTextStringCodingScheme() ;
            
            bRes = (result == (short)0x04) ;
            
        } catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 4 : Verify the TLV selected
        
        testCaseNb = (byte) 4 ;
        
        bRes = false ;
        try {
            
            // Get length of current TLV
            result = proRespHdlr.getValueLength() ;
            
            bRes = (result == (short)0x02) ;
            
        } catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 5 : Successfull call
        
        testCaseNb = (byte) 5 ;
        
        bRes = false ;
        try {
            
            // Build and send a GET INPUT proactive command
            proHdlr.initGetInput((byte)0x00, DCS_8_BIT_DATA, TEXT, (short)0, (short)TEXT.length, (short)0x00, (short)0xFF) ;
        
            proHdlr.send() ;
            
            // Get the response
            proRespHdlr = ProactiveResponseHandler.getTheHandler() ;
            
            result = proRespHdlr.getTextStringCodingScheme() ;
            
            bRes = (result == (short)0x00) ;
            
        } catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 6 : Verify the TLV selected
        
        testCaseNb = (byte) 6 ;
        
        bRes = false ;
        try {
            
            // Get length of current TLV
            result = proRespHdlr.getValueLength() ;
            
            bRes = (result == (short)0x03) ;
            
        } catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 7 : Successfull call
        
        testCaseNb = (byte) 7 ;
        
        bRes = false ;
        try {
            
            // Build and send a GET INPUT proactive command
            proHdlr.initGetInput((byte)0x00, DCS_8_BIT_DATA, TEXT, (short)0, (short)TEXT.length, (short)0x00, (short)0xFF) ;
        
            proHdlr.send() ;
            
            // Get the response
            proRespHdlr = ProactiveResponseHandler.getTheHandler() ;
            
            result = proRespHdlr.getTextStringCodingScheme() ;
            
            bRes = (result == (short)0x08) ;
            
        } catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 8 : Verify the TLV selected
        
        testCaseNb = (byte) 8 ;
        
        bRes = false ;
        try {
            
            // Get length of current TLV
            result = proRespHdlr.getValueLength() ;
            
            bRes = (result == (short)0x7F) ;
            
        } catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 9 : Successfull call
        
        testCaseNb = (byte) 9 ;
        
        bRes = false ;
        try {
            
            // Build and send a GET INPUT proactive command
            proHdlr.initGetInput((byte)0x00, DCS_8_BIT_DATA, TEXT, (short)0, (short)TEXT.length, (short)0x00, (short)0xFF) ;
        
            proHdlr.send() ;
            
            // Get the response
            proRespHdlr = ProactiveResponseHandler.getTheHandler() ;
            
            result = proRespHdlr.getTextStringCodingScheme() ;
            
            bRes = (result == (short)0x04) ;
            
        } catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 10 : Verify the TLV selected
        
        testCaseNb = (byte) 10 ;
        
        bRes = false ;
        try {
            
            // Get length of current TLV
            result = proRespHdlr.getValueLength() ;
            
            bRes = (result == (short)0x80) ;
            
        } catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 11 : Successfull call
        
        testCaseNb = (byte) 11 ;
        
        bRes = false ;
        try {
            
            // Build and send a GET INPUT proactive command
            proHdlr.initGetInput((byte)0x00, DCS_8_BIT_DATA, TEXT, (short)0, (short)TEXT.length, (short)0x00, (short)0xFF) ;
        
            proHdlr.send() ;
            
            // Get the response
            proRespHdlr = ProactiveResponseHandler.getTheHandler() ;
            
            result = proRespHdlr.getTextStringCodingScheme() ;
            
            bRes = (result == (short)0x08) ;
            
        } catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 12 : Verify the TLV selected
        
        testCaseNb = (byte) 12 ;
        
        bRes = false ;
        try {
            
            // Get length of current TLV
            result = proRespHdlr.getValueLength() ;
            
            bRes = (result == (short)0xF0) ;
            
        } catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 13 : Successfull call
        // Terminal Response with 2 Text String TLV
        
        testCaseNb = (byte) 13 ;
        
        bRes = false ;
        try {
            
            // Build and send a GET INPUT proactive command
            proHdlr.initGetInput((byte)0x00, DCS_8_BIT_DATA, TEXT, (short)0, (short)TEXT.length, (short)0x00, (short)0xFF) ;
        
            proHdlr.send() ;
            
            // Get the response
            proRespHdlr = ProactiveResponseHandler.getTheHandler() ;
            
            result = proRespHdlr.getTextStringCodingScheme() ;
            
            bRes = (result == (short)0x04) ;
            
        } catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 14 : Verify the TLV selected
        
        testCaseNb = (byte) 14 ;
        
        bRes = false ;
        try {
            
            // Get length of current TLV
            result = proRespHdlr.getValueLength() ;
            
            bRes = (result == (short)0x02) ;
            
        } catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;



    }
}
